@@ -562,6 +562,47 @@ private Q_SLOTS:
         ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/data/timeline/") + baseName + QLatin1String(".json")));
         QVERIFY(vp.verify(&model));
     }
+
+    void testCurrentReservation()
+    {
+        ReservationManager resMgr;
+        Test::clearAll(&resMgr);
+        ApplicationController ctrl;
+        ctrl.setReservationManager(&resMgr);
+
+        TimelineModel model;
+        model.setCurrentDateTime(QDateTime({2017, 8, 1}, {23, 0}, QTimeZone("Europe/Zurich")));
+        QAbstractItemModelTester tester(&model);
+        model.setReservationManager(&resMgr);
+
+        QSignalSpy currentResChangedSpy(&model, &TimelineModel::currentBatchChanged);
+
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
+        QCOMPARE(model.rowCount(), 13);
+        QVERIFY(!currentResChangedSpy.empty());
+
+        model.setCurrentDateTime(QDateTime({2017, 8, 1}, {23, 0}, QTimeZone("Europe/Zurich")));
+        QVERIFY(model.currentBatchId().isEmpty());
+
+        model.setCurrentDateTime(QDateTime({2017, 9, 9}, {23, 0}, QTimeZone("Europe/Zurich")));
+        QVERIFY(!model.currentBatchId().isEmpty());
+        QCOMPARE(model.currentBatchId(), model.index(1, 0).data(TimelineModel::BatchIdRole).toString());
+
+        model.setCurrentDateTime(QDateTime({2017, 9, 10}, {14, 0}, QTimeZone("Europe/Zurich")));
+        QCOMPARE(model.index(2, 0).data(TimelineModel::ElementTypeRole), TimelineElement::TrainTrip);
+        QVERIFY(!model.currentBatchId().isEmpty());
+        QCOMPARE(model.currentBatchId(), model.index(2, 0).data(TimelineModel::BatchIdRole).toString());
+
+        model.setCurrentDateTime(QDateTime({2017, 9, 10}, {14, 5}, QTimeZone("Europe/Zurich")));
+        QVERIFY(!model.currentBatchId().isEmpty());
+        QCOMPARE(model.currentBatchId(), model.index(3, 0).data(TimelineModel::BatchIdRole).toString());
+
+        model.setCurrentDateTime(QDateTime({2017, 9, 10}, {20, 0}, QTimeZone("Europe/Zurich")));
+        QVERIFY(model.currentBatchId().isEmpty());
+
+        model.setCurrentDateTime(QDateTime({2019, 1, 1}, {0, 0}, QTimeZone("Europe/Zurich")));
+        QVERIFY(model.currentBatchId().isEmpty());
+    }
 };
 
 QTEST_GUILESS_MAIN(TimelineModelTest)
