@@ -5,6 +5,7 @@
 */
 
 #include "timelinemodel.h"
+#include "constants.h"
 #include "locationhelper.h"
 #include "locationinformation.h"
 #include "pkpassmanager.h"
@@ -75,6 +76,7 @@ TimelineModel::TimelineModel(QObject *parent)
     : QAbstractListModel(parent)
 {
     connect(&m_dayUpdateTimer, &QTimer::timeout, this, &TimelineModel::dayChanged);
+    m_dayUpdateTimer.setTimerType(Qt::VeryCoarseTimer);
     m_dayUpdateTimer.setSingleShot(true);
     m_dayUpdateTimer.setInterval((QTime::currentTime().secsTo({23, 59, 59}) + 1) * 1000);
     m_dayUpdateTimer.start();
@@ -88,6 +90,11 @@ TimelineModel::TimelineModel(QObject *parent)
         m_todayEmpty = !m_todayEmpty;
         Q_EMIT dataChanged(idx, idx);
     });
+
+    connect(&m_currentBatchTimer, &QTimer::timeout, this, &TimelineModel::currentBatchChanged);
+    connect(&m_currentBatchTimer, &QTimer::timeout, this, &TimelineModel::scheduleCurrentBatchTimer);
+    m_currentBatchTimer.setTimerType(Qt::VeryCoarseTimer);
+    m_currentBatchTimer.setSingleShot(true);
 }
 
 TimelineModel::~TimelineModel() = default;
@@ -135,6 +142,9 @@ void TimelineModel::setReservationManager(ReservationManager* mgr)
 
     updateInformationElements();
     Q_EMIT todayRowChanged();
+
+    scheduleCurrentBatchTimer();
+    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::setWeatherForecastManager(WeatherForecastManager* mgr)
@@ -195,16 +205,9 @@ QVariant TimelineModel::data(const QModelIndex& index, int role) const
 
     const auto &elem = m_elements.at(index.row());
     switch (role) {
-        case SectionHeader:
-        {
-            if (elem.dt.isNull()) {
-                return {};
-            }
-            if (elem.dt.date() == today()) {
-                return i18n("Today");
-            }
-            return i18nc("weekday, date", "%1, %2", QLocale().dayName(elem.dt.date().dayOfWeek(), QLocale::LongFormat), QLocale().toString(elem.dt.date(), QLocale::ShortFormat));
-        }
+        case SectionHeaderRole:
+            // see TimelineSectionDelegateController
+            return elem.dt.date().toString(Qt::ISODate);
         case BatchIdRole:
             return elem.isReservation() ? elem.batchId() : QString();
         case ElementTypeRole:
@@ -254,6 +257,8 @@ QVariant TimelineModel::data(const QModelIndex& index, int role) const
                 return elem.content();
             }
             break;
+        case StartDateTimeRole:
+            return elem.dt;
     }
     return {};
 }
@@ -261,7 +266,7 @@ QVariant TimelineModel::data(const QModelIndex& index, int role) const
 QHash<int, QByteArray> TimelineModel::roleNames() const
 {
     auto names = QAbstractListModel::roleNames();
-    names.insert(SectionHeader, "sectionHeader");
+    names.insert(SectionHeaderRole, "sectionHeader");
     names.insert(BatchIdRole, "batchId");
     names.insert(ElementTypeRole, "type");
     names.insert(TodayEmptyRole, "isTodayEmpty");
@@ -295,6 +300,9 @@ void TimelineModel::batchAdded(const QString &resId)
     updateInformationElements();
     updateTransfersForBatch(resId);
     Q_EMIT todayRowChanged();
+
+    scheduleCurrentBatchTimer();
+    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::insertElement(TimelineElement &&elem)
@@ -343,6 +351,9 @@ void TimelineModel::batchChanged(const QString &resId)
     }
 
     updateInformationElements();
+
+    scheduleCurrentBatchTimer();
+    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::batchRenamed(const QString& oldBatchId, const QString& newBatchId)
@@ -405,6 +416,9 @@ void TimelineModel::batchRemoved(const QString &resId)
     }
 
     updateInformationElements();
+
+    scheduleCurrentBatchTimer();
+    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::dayChanged()
@@ -414,6 +428,9 @@ void TimelineModel::dayChanged()
 
     m_dayUpdateTimer.setInterval((QTime::currentTime().secsTo({23, 59, 59}) + 1) * 1000);
     m_dayUpdateTimer.start();
+
+    scheduleCurrentBatchTimer();
+    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::updateTodayMarker()
@@ -665,6 +682,8 @@ void TimelineModel::setCurrentDateTime(const QDateTime &dt)
     if (dayDiffers && !m_elements.empty()) {
         dayChanged();
     }
+
+    scheduleCurrentBatchTimer();
 }
 
 void TimelineModel::tripGroupAdded(const QString& groupId)
@@ -746,3 +765,132 @@ void TimelineModel::transferRemoved(const QString &resId, Transfer::Alignment al
 
     Q_EMIT todayRowChanged();
 }
+
+static bool isSelectableElement(const TimelineElement &elem)
+{
+    return elem.isReservation() && elem.rangeType == TimelineElement::SelfContained;
+}
+
+QString TimelineModel::currentBatchId() const
+{
+    if (m_elements.empty()) {
+        return {};
+    }
+
+    // find the next reservation
+    auto it = std::lower_bound(m_elements.begin(), m_elements.end(), now());
+    for (; it != m_elements.end() && !isSelectableElement(*it); ++it) {}
+
+    QString nextResId;
+    QDateTime nextStartTime;
+    if (it != m_elements.end() && now().secsTo((*it).dt) < Constants::CurrentBatchLeadingMargin.count()) {
+        nextResId = (*it).batchId();
+        nextStartTime = (*it).dt;
+    }
+
+    // find the previous or current reservation
+    if (it != m_elements.begin()) {
+        --it;
+        for (; it != m_elements.begin() && (it == m_elements.end() || !isSelectableElement(*it)); --it) {}
+    }
+
+    const auto resId = (*it).batchId();
+    const auto res = m_resMgr->reservation(resId);
+    auto endTime = SortUtil::endDateTime(res);
+    if (endTime.secsTo(now()) > Constants::CurrentBatchTrailingMargin.count()) {
+        endTime = {};
+    }
+
+    // only one side found
+    if (!endTime.isValid()) {
+        return nextResId;
+    }
+    if (!nextStartTime.isValid()) {
+        return resId;
+    }
+
+    // (*it) is still active
+    if (endTime >= now()) {
+        return resId;
+    }
+
+    // take the one that is closer
+    return endTime.secsTo(now()) < now().secsTo(nextStartTime) ? resId : nextResId;
+}
+
+void TimelineModel::scheduleCurrentBatchTimer()
+{
+    if (m_elements.empty()) {
+        return;
+    }
+
+    // we need the smallest valid time > now() of any of the following:
+    // - end time of the current element + margin
+    // - start time - margin of the next res
+    // - end time of current + start time of next - endtime of current / 2
+    // - end time of the next element (in case we are in the leading margin already)
+
+    QDateTime triggerTime;
+    const auto updateTriggerTime = [&triggerTime, this](const QDateTime &dt) {
+        if (!dt.isValid() || dt <= now()) {
+            return;
+        }
+        if (!triggerTime.isValid()) {
+            triggerTime = dt;
+        } else {
+            triggerTime = std::min(triggerTime, dt);
+        }
+    };
+
+    // find the next reservation
+    auto it = std::lower_bound(m_elements.begin(), m_elements.end(), now());
+    for (; it != m_elements.end() && !isSelectableElement(*it); ++it) {}
+
+    QDateTime nextStartTime;
+    if (it != m_elements.end()) {
+        nextStartTime = (*it).dt;
+        updateTriggerTime(nextStartTime.addSecs(-Constants::CurrentBatchLeadingMargin.count()));
+        updateTriggerTime(SortUtil::endDateTime(m_resMgr->reservation((*it).batchId())));
+    }
+
+    // find the previous or current reservation
+    if (it != m_elements.begin()) {
+        --it;
+        for (; it != m_elements.begin() && (it == m_elements.end() || !isSelectableElement(*it)); --it) {}
+    }
+
+    const auto res = m_resMgr->reservation((*it).batchId());
+    auto endTime = SortUtil::endDateTime(res);
+    updateTriggerTime(endTime.addSecs(Constants::CurrentBatchTrailingMargin.count()));
+
+    if (nextStartTime.isValid() && endTime.isValid()) {
+        updateTriggerTime(endTime.addSecs(endTime.secsTo(nextStartTime) / 2));
+    }
+
+    // QTimer only has 31bit for its msec interval, so don't schedule beyond a day
+    // for longer distances we re-run this in the midnight timer above
+    if (triggerTime.isValid() && triggerTime.date() == today()) {
+        m_currentBatchTimer.setInterval(std::chrono::seconds(std::max<qint64>(60, now().secsTo(triggerTime))));
+        m_currentBatchTimer.start();
+    }
+}
+
+QVariant TimelineModel::locationAtTime(const QDateTime& dt) const
+{
+    auto it = std::lower_bound(m_elements.begin(), m_elements.end(), dt);
+    if (it == m_elements.begin()) {
+        return {};
+    }
+
+    for (--it; it != m_elements.begin(); --it) {
+        if (!(*it).isReservation()) {
+            continue;
+        }
+        const auto res = m_resMgr->reservation((*it).batchId());
+        if (!LocationUtil::isLocationChange(res)) {
+            continue;
+        }
+        return LocationUtil::arrivalLocation(res);
+    }
+    return {};
+}
